==> list-fn-0.5.0.crate: list-fn-0.5.0/src/iter.rs <==
use super::*;
use take_mut::take;

/// Any `&mut Iterator` is a `ListFn<End = Self>`.
impl<I: Iterator> ListFn for &mut I {
    type Item = I::Item;
    type End = Self;
    /// Converts an iterator into a list.
    fn list(self) -> List<Self> {
        match self.next() {
            Option::None => List::End(self),
            Option::Some(first) => List::Some(first, self),
        }
    }
}

impl<I: Iterator> ResultFn for &mut I {
    type Result = ();
    fn result(self) -> () {
        ()
    }
}

pub struct ListIterator<S: ListFn<End = S>>(S);

impl<S: ListFn<End = S>> Iterator for ListIterator<S> {
    type Item = S::Item;
    fn next(&mut self) -> Option<Self::Item> {
        let mut result = None;
        take(&mut self.0, |list| match list.list() {
            List::End(end) => end,
            List::Some(first, next) => {
                result = Some(first);
                next
            }
        });
        result
    }
}

/// Note: we can't use the standard std::iter::IntoIterator because it has
/// a conflicting implementation.
pub trait IntoIter: ListFn<End = Self> {
    /// Converts a list to an iterator.
    fn iter(self) -> ListIterator<Self> {
        ListIterator(self)
    }
}

impl<S: ListFn<End = Self>> IntoIter for S {}

pub enum ListIteratorWrap<L: ListFn> {
    List(L),
    End(L::End),
}

impl<L: ListFn> Iterator for ListIteratorWrap<L> {
    type Item = L::Item;
    fn next(&mut self) -> Option<Self::Item> {
        let mut result = None;
        take(self, |wrap| match wrap {
            ListIteratorWrap::List(list) => match list.list() {
                List::End(end) => ListIteratorWrap::End(end),
                List::Some(first, next) => {
                    result = Some(first);
                    ListIteratorWrap::List(next)
                }
            },
            end => end,
        });
        result
    }
}

pub trait IntoIterWrap: ListFn {
    fn iter_wrap(self) -> ListIteratorWrap<Self> {
        ListIteratorWrap::List(self)
    }
}

impl<L: ListFn> IntoIterWrap for L {}

==> list-fn-0.7.0.crate: list-fn-0.7.0/src/iter.rs <==
use super::*;
use take_mut::take;

/// Any `&mut Iterator` is a `ListFn<End = Self>`.
impl<I: Iterator> ListFn for &mut I {
    type Item = I::Item;
    type End = Self;
    /// Converts an iterator into a list.
    fn state(self) -> ListState<Self> {
        match self.next() {
            Option::None => ListState::End(self),
            Option::Some(first) => ListState::Some(first, self),
        }
    }
}

impl<I: Iterator> ResultFn for &mut I {
    type Result = ();
    fn result(self) -> () {
        ()
    }
}

pub struct ListIterator<S: ListFn<End = S>>(S);

impl<S: ListFn<End = S>> Iterator for ListIterator<S> {
    type Item = S::Item;
    fn next(&mut self) -> Option<Self::Item> {
        let mut result = None;
        take(&mut self.0, |list| match list.state() {
            ListState::End(end) => end,
            ListState::Some(first, next) => {
                result = Some(first);
                next
            }
        });
        result
    }
}

/// Note: we can't use the standard std::iter::IntoIterator because it has
/// a conflicting implementation.
pub trait IntoIter: ListFn<End = Self> {
    /// Converts a list to an iterator.
    fn iter(self) -> ListIterator<Self> {
        ListIterator(self)
    }
}

impl<S: ListFn<End = Self>> IntoIter for S {}

pub enum ListIteratorWrap<L: ListFn> {
    List(L),
    End(L::End),
}

impl<L: ListFn> Iterator for ListIteratorWrap<L> {
    type Item = L::Item;
    fn next(&mut self) -> Option<Self::Item> {
        let mut result = None;
        take(self, |wrap| match wrap {
            ListIteratorWrap::List(list) => match list.state() {
                ListState::End(end) => ListIteratorWrap::End(end),
                ListState::Some(first, next) => {
                    result = Some(first);
                    ListIteratorWrap::List(next)
                }
            },
            end => end,
        });
        result
    }
}

pub trait IntoIterWrap: ListFn {
    fn iter_wrap(self) -> ListIteratorWrap<Self> {
        ListIteratorWrap::List(self)
    }
}

impl<L: ListFn> IntoIterWrap for L {}
